import { IConfig } from "./doc/IConfig";
import { IConfigLayer } from "./doc/IConfigLayer";
import { IConfigProfile } from "./doc/IConfigProfile";
import { IConfigOpts } from "./doc/IConfigOpts";
import { IConfigSecure } from "./doc/IConfigSecure";
import { IConfigVault } from "./doc/IConfigVault";
import { ConfigLayers, ConfigPlugins, ConfigProfiles, ConfigSecure } from "./api";
/**
 * The Config class provides facilities for reading and writing team
 * configuration files. It is used by Imperative to perform low-level
 * operations on a team configuration. The intent is that consumer
 * apps will not typically use the Config class, since end-users are
 * expected to write team configuration files by directly editing them
 * in an editor like VSCode.
 */
export declare class Config {
    opts?: IConfigOpts;
    /**
     * The trailing portion of a shared config file name
     */
    private static readonly END_OF_TEAM_CONFIG;
    /**
     * The trailing portion of a user-specific config file name
     */
    private static readonly END_OF_USER_CONFIG;
    /**
     * App name used in config filenames (e.g., *my_cli*.config.json)
     * @internal
     */
    mApp: string;
    /**
     * List to store each of the config layers enumerated in `layers` enum
     * @internal
     */
    mLayers: IConfigLayer[];
    /**
     * Directory where global config files are located. Defaults to `~/.appName`.
     * @internal
     */
    mHomeDir: string;
    /**
     * Directory where project config files are located. Defaults to working directory.
     * @internal
     */
    mProjectDir: string;
    /**
     * Currently active layer whose properties will be manipulated
     * @internal
     */
    mActive: {
        user: boolean;
        global: boolean;
    };
    /**
     * Vault object with methods for loading and saving secure credentials
     * @internal
     */
    mVault: IConfigVault;
    /**
     * Secure properties object stored in credential vault
     * @internal
     */
    mSecure: IConfigSecure;
    /**
     * Constructor for Config class. Don't use this directly. Await `Config.load` instead.
     * @param opts Options to control how Config class behaves
     * @private
     */
    private constructor();
    /**
     * Return a Config interface with required fields initialized as empty.
     */
    static empty(): IConfig;
    /**
     * Load config files from disk and secure properties from vault.
     * @param app App name used in config filenames (e.g., *my_cli*.config.json)
     * @param opts Options to control how Config class behaves
     */
    static load(app: string, opts?: IConfigOpts): Promise<Config>;
    /**
     * Save config files to disk and store secure properties in vault.
     * @param allLayers Specify false to save only the active config layer
     */
    save(allLayers?: boolean): Promise<void>;
    /**
     * Get absolute file path for a config layer.
     * For project config files, We search up from our current directory and
     * ignore the Zowe hone directory (in case our current directory is under
     * Zowe home.). For golbal config files we only retrieve config files
     * from the Zowe home directory.
     *
     * @internal
     * @param layer Enum value for config layer
     */
    private layerPath;
    /**
     * Access the config API for manipulating profiles, plugins, layers, and secure values.
     */
    get api(): {
        profiles: ConfigProfiles;
        plugins: ConfigPlugins;
        layers: ConfigLayers;
        secure: ConfigSecure;
    };
    /**
     * True if any config layers exist on disk, otherwise false.
     */
    get exists(): boolean;
    /**
     * List of absolute file paths for all config layers.
     */
    get paths(): string[];
    /**
     * List of all config layers.
     * Returns a clone to prevent accidental edits of the original object.
     */
    get layers(): IConfigLayer[];
    /**
     * List of properties across all config layers.
     * Returns a clone to prevent accidental edits of the orignal object.
     */
    get properties(): IConfig;
    /**
     * App name used in config filenames (e.g., *my_cli*.config.json)
     */
    get appName(): string;
    /**
     * Filename used for config JSONC files
     */
    get configName(): string;
    /**
     * Filename used for user config JSONC files
     */
    get userConfigName(): string;
    /**
     * Filename used for config schema JSON files
     */
    get schemaName(): string;
    /**
     * Search up the directory tree for the directory containing the
     * specified config file.
     *
     * @param file Contains the name of the desired config file
     * @param opts.ignoreDirs Contains an array of directory names to be
     *        ignored (skipped) during the search.
     * @param opts.startDir Contains the name of the directory where the
     *        search should be started. Defaults to working directory.
     *
     * @returns The full path name to config file or null if not found.
     */
    static search(file: string, opts?: {
        ignoreDirs?: string[];
        startDir?: string;
    }): string;
    /**
     * The properties object with secure values masked.
     * @type {IConfig}
     * @memberof Config
     */
    get maskedProperties(): IConfig;
    /**
     * Set value of a property in the active config layer.
     * TODO: more validation
     *
     * @param path Property path
     * @param value Property value
     * @param opts Include `secure: true` to store the property securely
     */
    set(path: string, value: any, opts?: {
        secure?: boolean;
    }): void;
    /**
     * Unset value of a property in the active config layer.
     * @param path Property path
     * @param opts Include `secure: false` to preserve property in secure array
     */
    delete(path: string, opts?: {
        secure?: boolean;
    }): void;
    /**
     * Set the $schema value at the top of the config JSONC.
     * Also save the schema to disk if an object is provided.
     * @param schema The URI of JSON schema, or a schema object to use
     */
    setSchema(schema: string | object): void;
    /**
     * Merge the properties from multiple layers into a single Config object.
     *
     * @internal
     * @param maskSecure Indicates whether we should mask off secure properties.
     *
     * @returns The resulting Config object
     */
    private layerMerge;
    /**
     * Obtain the profiles object for a specified layer object.
     *
     * @internal
     * @param layer The layer for which we want the profiles.
     * @param maskSecure If true, we will mask the values of secure properties.
     *
     * @returns The resulting profile object
     */
    layerProfiles(layer: IConfigLayer, maskSecure?: boolean): {
        [key: string]: IConfigProfile;
    };
    /**
     * Find the layer with the specified user and global properties.
     *
     * @internal
     * @param user True specifies that you want the user layer.
     * @param global True specifies that you want the layer at the global level.
     *
     * @returns The desired layer object. Null if no layer matches.
     */
    findLayer(user: boolean, global: boolean): IConfigLayer;
    /**
     * Obtain the layer object that is currently active.
     *
     * @internal
     *
     * @returns The active layer object
     */
    layerActive(): IConfigLayer;
    /**
     * Form the path name of the team config file to display in messages.
     * Always return the team name (not the user name).
     * If the a team configuration is active, return the full path to the
     * config file.
     *
     * @param options - a map containing option properties. Currently, the only
     *                  property supported is a boolean named addPath.
     *                  {addPath: true | false}
     *
     * @returns The path (if requested) and file name of the team config file.
     */
    formMainConfigPathNm(options: any): string;
}
