"use strict";
/*
* This program and the accompanying materials are made available under the terms of the
* Eclipse Public License v2.0 which accompanies this distribution, and is available at
* https://www.eclipse.org/legal/epl-v20.html
*
* SPDX-License-Identifier: EPL-2.0
*
* Copyright Contributors to the Zowe Project.
*
*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConfigLayers = void 0;
const fs = require("fs");
const node_path = require("path");
const JSONC = require("comment-json");
const lodash = require("lodash");
const error_1 = require("../../../error");
const ConfigConstants_1 = require("../ConfigConstants");
const ConfigApi_1 = require("./ConfigApi");
/**
 * API Class for manipulating config layers.
 */
class ConfigLayers extends ConfigApi_1.ConfigApi {
    // _______________________________________________________________________
    /**
     * Read a config layer from disk into memory for application use.
     *
     * @param opts The user and global flags that indicate which of the four
     *             config files (aka layers) is to be read.
     */
    read(opts) {
        return __awaiter(this, void 0, void 0, function* () {
            // Attempt to populate the layer
            const layer = opts ? this.mConfig.findLayer(opts.user, opts.global) : this.mConfig.layerActive();
            if (fs.existsSync(layer.path)) {
                let fileContents;
                try {
                    fileContents = fs.readFileSync(layer.path);
                }
                catch (e) {
                    throw new error_1.ImperativeError({ msg: `An error was encountered while trying to read the file '${layer.path}'.` +
                            `\nError details: ${e.message}`,
                        suppressDump: true });
                }
                try {
                    layer.properties = JSONC.parse(fileContents.toString());
                    layer.exists = true;
                }
                catch (e) {
                    throw new error_1.ImperativeError({ msg: `Error parsing JSON in the file '${layer.path}'.\n` +
                            `Please check this configuration file for errors.\nError details: ${e.message}\nLine ${e.line}, Column ${e.column}`,
                        suppressDump: true });
                }
            }
            // Populate any undefined defaults
            layer.properties.defaults = layer.properties.defaults || {};
            layer.properties.profiles = layer.properties.profiles || {};
            layer.properties.plugins = layer.properties.plugins || [];
        });
    }
    // _______________________________________________________________________
    /**
     * Write a config layer to disk.
     *
     * @param opts The user and global flags that indicate which of the four
     *             config files (aka layers) is to be written.
     */
    write(opts) {
        return __awaiter(this, void 0, void 0, function* () {
            // TODO: should we prevent a write if there is no vault
            // TODO: specified and there are secure fields??
            // If fields are marked as secure
            const layer = opts ? this.mConfig.findLayer(opts.user, opts.global) : this.mConfig.layerActive();
            const layerCloned = JSONC.parse(JSONC.stringify(layer, null, ConfigConstants_1.ConfigConstants.INDENT));
            for (const path of this.mConfig.api.secure.secureFields(layer)) {
                const segments = path.split(".");
                let obj = layerCloned.properties;
                for (let x = 0; x < segments.length; x++) {
                    const segment = segments[x];
                    const v = obj[segment];
                    if (v == null)
                        break;
                    if (x === segments.length - 1) {
                        delete obj[segment];
                        break;
                    }
                    obj = obj[segment];
                }
            }
            // Write the layer
            try {
                fs.writeFileSync(layer.path, JSONC.stringify(layerCloned.properties, null, ConfigConstants_1.ConfigConstants.INDENT));
            }
            catch (e) {
                throw new error_1.ImperativeError({ msg: `error writing "${layer.path}": ${e.message}` });
            }
            layer.exists = true;
        });
    }
    // _______________________________________________________________________
    /**
     * Select which layer is the currently active layer.
     *
     * @param user True if you want the user layer.
     * @param global True if you want the global layer.
     * @param inDir The directory to which you want to set the file path
     *              for this layer.
     */
    activate(user, global, inDir) {
        this.mConfig.mActive.user = user;
        this.mConfig.mActive.global = global;
        if (inDir != null) {
            const layer = this.mConfig.layerActive();
            layer.path = node_path.join(inDir, node_path.basename(layer.path));
            this.read();
        }
    }
    // _______________________________________________________________________
    /**
     * Gets a json object that represents the currently active layer.
     *
     * @returns The json object
     */
    get() {
        // Note: Add indentation to allow comments to be accessed via config.api.layers.get(), otherwise use layerActive()
        // return JSONC.parse(JSONC.stringify(this.mConfig.layerActive(), null, ConfigConstants.INDENT));
        return JSONC.parse(JSONC.stringify(this.mConfig.layerActive(), null, ConfigConstants_1.ConfigConstants.INDENT));
    }
    // _______________________________________________________________________
    /**
     * Set the the currently active layer to the supplied json object.
     *
     * @param user True if you want the user layer.
     */
    set(cnfg) {
        for (const i in this.mConfig.mLayers) {
            if (this.mConfig.mLayers[i].user === this.mConfig.mActive.user &&
                this.mConfig.mLayers[i].global === this.mConfig.mActive.global) {
                this.mConfig.mLayers[i].properties = cnfg;
                this.mConfig.mLayers[i].properties.defaults = this.mConfig.mLayers[i].properties.defaults || {};
                this.mConfig.mLayers[i].properties.profiles = this.mConfig.mLayers[i].properties.profiles || {};
                this.mConfig.mLayers[i].properties.plugins = this.mConfig.mLayers[i].properties.plugins || [];
            }
        }
    }
    // _______________________________________________________________________
    /**
     * Merge properties from the supplied Config object into the active layer.
     * If dryRun is specified, merge is applied to a copy of the layer and returned.
     * If dryRun is not specified, merge is applied directly to the layer and nothing is returned.
     *
     * @param cnfg The Config object to merge.
     * @returns The merged config layer or void
     */
    merge(cnfg, dryRun = false) {
        let layer;
        if (dryRun) {
            layer = JSONC.parse(JSONC.stringify(this.mConfig.layerActive(), null, ConfigConstants_1.ConfigConstants.INDENT));
        }
        else {
            layer = this.mConfig.layerActive();
        }
        layer.properties.profiles = lodash.mergeWith(cnfg.profiles, layer.properties.profiles, (obj, src) => {
            if (lodash.isArray(obj) && lodash.isArray(src)) {
                const temp = JSONC.parse(JSONC.stringify(obj, null, ConfigConstants_1.ConfigConstants.INDENT));
                src.forEach((val, idx) => {
                    if (!temp.includes(val)) {
                        temp.splice(idx, 0, val);
                    }
                });
                return temp;
            }
        });
        layer.properties.defaults = lodash.merge(cnfg.defaults, layer.properties.defaults);
        for (const pluginName of cnfg.plugins) {
            if (!layer.properties.plugins.includes(pluginName)) {
                layer.properties.plugins.push(pluginName);
            }
        }
        if (dryRun) {
            return layer;
        }
    }
}
exports.ConfigLayers = ConfigLayers;
//# sourceMappingURL=ConfigLayers.js.map