import { IConfigLoadedProfile } from "../doc/IConfigLoadedProfile";
import { IConfigProfile } from "../doc/IConfigProfile";
import { ConfigApi } from "./ConfigApi";
/**
 * API Class for manipulating config profiles.
 */
export declare class ConfigProfiles extends ConfigApi {
    /**
     * Set a profile object at the location identified by the path
     * within the currently active layer.
     *
     * @param path The dotted path of the location in which to set the profile.
     * @param profile The JSON profile object to set into the specified location,
     */
    set(path: string, profile: IConfigProfile): void;
    /**
     * Get the profile object located at the specified location.
     *
     * TODO: If asked for inner layer profile, if profile doesn't exist,
     *       returns outer layer profile values (bug?)
     *
     * @param path The dotted path of the location at which to set the profile.
     */
    get(path: string): {
        [key: string]: string;
    };
    /**
     * Reports whether or not a profile exists at the specified location.
     *
     * @param path The dotted path of desired location.
     *
     * @returns True if a profile exists. False otherwise.
     */
    exists(path: string): boolean;
    /**
     * Load the properties and sub-profiles of the profile at the
     * specified location.
     *
     * @param path The dotted path of desired location.
     *
     * @returns The desired profile object. null if the path does not exist.
     */
    load(path: string): IConfigLoadedProfile;
    /**
     * Set the default value for the specified type of profile within
     * the currently active layer.
     *
     * @param profileType The name of the desired type of profile (like zosmf).
     * @param value The dotted node path to the profile (like ca32.zosmf).
     */
    defaultSet(profileType: string, value: string): void;
    /**
     * Get the profile contents for the default profile of the specified type
     * of profile within the currently active layer.
     *
     * @param profileType The name of the desired type of profile (like zosmf).
     *
     * @returns An object containing the desired profile,
     *          for example {"host": "lpar.your.domain.net", port: 1234}
     */
    defaultGet(profileType: string): {
        [key: string]: string;
    };
    /**
     * Expands a short path into an expanded path.
     *
     * @param shortPath The short path.
     *
     * @returns The expanded path.
     *
     */
    expandPath(shortPath: string): string;
    /**
     * Load the properties and sub-profiles of the profile at the
     * specified location.
     *
     * Place secure values into their expected locations within the profile.
     *
     * TODO: Does this need to recurse up through nested profiles?
     *
     * @param path The dotted path of desired location.
     *
     * @returns The desired profile object. null if the path does not exist.
     */
    private loadProfile;
    /**
     * Build the set of properties contained within a set of nested profiles.
     *
     * @param path The dotted path of desired location.
     * @param profiles A set of nested profile objects.
     *
     * @returns The desired profile object. An empty object if profiles is empty.
     */
    private buildProfile;
    /**
     * Find a profile at a specified location from within a set of
     * nested profiles.
     *
     * @param path The dotted path of desired location.
     * @param profiles A set of nested profile objects.
     *
     * @returns The profile object that was found. null if not found.
     */
    private findProfile;
}
