"use strict";
/*
* This program and the accompanying materials are made available under the terms of the
* Eclipse Public License v2.0 which accompanies this distribution, and is available at
* https://www.eclipse.org/legal/epl-v20.html
*
* SPDX-License-Identifier: EPL-2.0
*
* Copyright Contributors to the Zowe Project.
*
*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.OverridesLoader = void 0;
const security_1 = require("../../security");
const path_1 = require("path");
const settings_1 = require("../../settings");
const utilities_1 = require("../../utilities");
const logger_1 = require("../../logger");
/**
 * Imperative-internal class to load overrides
 * You should not need to call this from your CLI.
 */
class OverridesLoader {
    /**
     * Apply overrides to all applicable facilities and use our defaults where
     * an override is not provided.
     *
     * @param {IImperativeConfig} config - the current {@link Imperative#loadedConfig}
     * @param {any} packageJson - the current package.json
     */
    static load(config, packageJson) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            // Initialize the Credential Manager
            yield (((_a = utilities_1.ImperativeConfig.instance.config) === null || _a === void 0 ? void 0 : _a.exists) ? this.loadCredentialManager : this.loadCredentialManagerOld)(config, packageJson);
        });
    }
    /**
     * Load the baked-in zowe CredentialManager and initialize it.
     * If we need to reinstate 3rd party overrides, delete this function and
     * rename loadCredentialManagerOld.
     *
     * @internal
     * @param {IImperativeConfig} config - the current {@link Imperative#loadedConfig}
     * @param {any} packageJson - the current package.json
     */
    static loadCredentialManager(config, packageJson) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            if (((_a = packageJson.dependencies) === null || _a === void 0 ? void 0 : _a.keytar) != null || ((_b = packageJson.optionalDependencies) === null || _b === void 0 ? void 0 : _b.keytar) != null) {
                // Load the CredentialManager built into Imperative for managing secure properties
                yield security_1.CredentialManagerFactory.initialize({
                    // The display name will be the plugin name that introduced the override OR it will default to the CLI name
                    displayName: config.productDisplayName || config.name,
                    // For overrides, the service could be the CLI name, but we do not override anymore.
                    // Null will use the service name of the built-in credential manager.
                    service: config.credentialServiceName || null,
                    // If the default is to be used, we won't implant the invalid credential manager.
                    // We do not use the invalid credential manager, since we no longer allow overrides.
                    invalidOnFailure: false
                });
            }
            yield OverridesLoader.loadSecureConfig();
        });
    }
    /**
     * Initialize the Credential Manager using the supplied override when provided.
     *
     * @param {IImperativeConfig} config - the current {@link Imperative#loadedConfig}
     * @param {any} packageJson - the current package.json
     */
    static loadCredentialManagerOld(config, packageJson) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const overrides = config.overrides;
            const ZOWE_CLI_PACKAGE_NAME = `@zowe/cli`;
            // The manager display name used to populate the "managed by" fields in profiles
            const displayName = (overrides.CredentialManager != null
                && settings_1.AppSettings.initialized
                && settings_1.AppSettings.instance.getNamespace("overrides") != null
                && settings_1.AppSettings.instance.get("overrides", "CredentialManager") != null
                && settings_1.AppSettings.instance.get("overrides", "CredentialManager") !== false) ?
                // App settings is configured - use the plugin name for the manager name
                settings_1.AppSettings.instance.get("overrides", "CredentialManager")
                :
                    // App settings is not configured - use the CLI display name OR the package name as the manager name
                    config.productDisplayName || config.name;
            // Initialize the credential manager if an override was supplied and/or keytar was supplied in package.json
            if (overrides.CredentialManager != null || ((_a = packageJson.dependencies) === null || _a === void 0 ? void 0 : _a.keytar) != null) {
                let Manager = overrides.CredentialManager;
                if (typeof overrides.CredentialManager === "string" && !path_1.isAbsolute(overrides.CredentialManager)) {
                    Manager = path_1.resolve(process.mainModule.filename, "../", overrides.CredentialManager);
                }
                yield security_1.CredentialManagerFactory.initialize({
                    // Init the manager with the override specified OR (if null) default to keytar
                    Manager,
                    // The display name will be the plugin name that introduced the override OR it will default to the CLI name
                    displayName,
                    // zowe cli will always add `Zowe` to it's list of service names
                    service: (config === null || config === void 0 ? void 0 : config.name) === ZOWE_CLI_PACKAGE_NAME ? security_1.DefaultCredentialManager.SVC_NAME : config === null || config === void 0 ? void 0 : config.name,
                    // If the default is to be used, we won't implant the invalid credential manager
                    invalidOnFailure: !(Manager == null)
                });
            }
            yield OverridesLoader.loadSecureConfig();
        });
    }
    /**
     * After the plugins and secure credentials are loaded, rebuild the configuration with the
     * secure values
     */
    static loadSecureConfig() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!security_1.CredentialManagerFactory.initialized)
                return;
            const vault = {
                load: ((key) => {
                    return security_1.CredentialManagerFactory.manager.load(key, true);
                }),
                save: ((key, value) => {
                    return security_1.CredentialManagerFactory.manager.save(key, value);
                })
            };
            try {
                yield utilities_1.ImperativeConfig.instance.config.api.secure.load(vault);
            }
            catch (err) {
                // Secure vault is optional since we can prompt for values instead
                logger_1.Logger.getImperativeLogger().warn(`Secure vault not enabled. Reason: ${err.message}`);
            }
        });
    }
}
exports.OverridesLoader = OverridesLoader;
//# sourceMappingURL=OverridesLoader.js.map