"use strict";
/*
* This program and the accompanying materials are made available under the terms of the
* Eclipse Public License v2.0 which accompanies this distribution, and is available at
* https://www.eclipse.org/legal/epl-v20.html
*
* SPDX-License-Identifier: EPL-2.0
*
* Copyright Contributors to the Zowe Project.
*
*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const utilities_1 = require("../../../../../utilities");
const config_1 = require("../../../../../config");
const ConfigBuilder_1 = require("../../../../../config/src/ConfigBuilder");
const security_1 = require("../../../../../security");
const ConfigUtils_1 = require("../../../../../config/src/ConfigUtils");
const OverridesLoader_1 = require("../../../OverridesLoader");
/**
 * Init config
 */
class InitHandler {
    /**
     * Process the command and input.
     *
     * @param {IHandlerParameters} params Parameters supplied by yargs
     *
     * @throws {ImperativeError}
     */
    process(params) {
        return __awaiter(this, void 0, void 0, function* () {
            this.params = params;
            // Load the config and set the active layer according to user options
            yield this.ensureCredentialManagerLoaded();
            const config = utilities_1.ImperativeConfig.instance.config;
            const configDir = params.arguments.globalConfig ? null : process.cwd();
            config.api.layers.activate(params.arguments.userConfig, params.arguments.globalConfig, configDir);
            const layer = config.api.layers.get();
            // // Protect against overwrite of the config
            // if (layer.exists && !params.arguments.update)
            //     throw new ImperativeError({ msg: `config "${layer.path}" already exists` });
            // Init as requested
            // if (this.arguments.url) {
            //     await this.initFromURL(config);
            // } else if (this.arguments.profile) {
            //     // TODO Should we remove old profile init code that prompts for values
            //     this.initProfile(config);
            // } else {
            //     await this.initWithSchema(config);
            // }
            yield this.initWithSchema(config, params.arguments.userConfig);
            if (params.arguments.prompt !== false && !security_1.CredentialManagerFactory.initialized && config.api.secure.secureFields().length > 0) {
                const warning = ConfigUtils_1.secureSaveError();
                params.response.console.log(utilities_1.TextUtils.chalk.yellow("Warning:\n") +
                    `${warning.message} Skipped prompting for credentials.\n\n${warning.additionalDetails}\n`);
            }
            // Write the active created/updated config layer
            yield config.save(false);
            params.response.console.log(`Saved config template to ${layer.path}`);
        });
    }
    /**
     * If CredentialManager was not already loaded by Imperative.init, load it
     * now before performing config operations in the init handler.
     */
    ensureCredentialManagerLoaded() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!security_1.CredentialManagerFactory.initialized) {
                yield OverridesLoader_1.OverridesLoader.loadCredentialManager(utilities_1.ImperativeConfig.instance.loadedConfig, utilities_1.ImperativeConfig.instance.callerPackageJson);
            }
        });
    }
    /**
     * Initialize a profile in the config
     * @param config The config
     */
    // private initProfile(config: Config) {
    //     const profile: IConfigProfile = { properties: {} };
    //     if (this.arguments.type != null) this.initProfileType(profile);
    //     config.api.profiles.set(this.arguments.profile, profile);
    // }
    /**
     * Initialize the profile using the type schema as a guide
     * @param config The config
     * @param profile The profile object to populate
     */
    // private async initProfileType(profile: IConfigProfile): Promise<void> {
    //     const schema = ImperativeConfig.instance.profileSchemas[this.arguments.type];
    //     if (schema == null)
    //         throw new ImperativeError({ msg: `profile type ${this.arguments.type} does not exist.` });
    //     // Use the schema to prompt for values
    //     profile.type = this.arguments.type;
    //     const secure: string[] = [];
    //     for (const [name, property] of Object.entries(schema.properties)) {
    //         const value: any = await this.promptForProp(property, name);
    //         // if secure, remember for the config set
    //         if (property.secure)
    //             secure.push(name);
    //         if (value != null) {
    //             profile.properties[name] = value;
    //         } else if (this.arguments.default && property.optionDefinition.defaultValue != null) {
    //             profile.properties[name] = property.optionDefinition.defaultValue;
    //         }
    //     }
    // }
    /**
     * Download/create the config from a URL
     * @param config The config
     */
    // private async initFromURL(config: Config): Promise<void> {
    //     const cnfg: IConfig = await this.download(this.arguments.url);
    //     config.api.layers.set(cnfg);
    // }
    /**
     * Download the config from a URL
     * @param url
     */
    // private download(url: string): Promise<IConfig> {
    //     // TODO Do we want to use node-fetch here?
    //     return new Promise<IConfig>((resolve, reject) => {
    //         https.get(url, (resp) => {
    //             let data = '';
    //             resp.on('data', (chunk) => { data += chunk; });
    //             resp.on('end', () => {
    //                 let cnfg;
    //                 let ok = false;
    //                 try {
    //                     cnfg = JSON.parse(data);
    //                     // TODO: additional validation?
    //                     ok = true;
    //                 } catch (e) {
    //                     reject(new ImperativeError({ msg: `unable to parse config: ${e.message}` }));
    //                 }
    //                 if (ok)
    //                     resolve(cnfg);
    //             });
    //         }).on("error", (err) => { reject(err); });
    //     });
    // }
    /**
     * Creates JSON template for config. Also creates a schema file in the same
     * folder alongside the config.
     * @param config Config object to be populated
     * @param user If true, properties will be left empty for user config
     */
    initWithSchema(config, user) {
        return __awaiter(this, void 0, void 0, function* () {
            // Build the schema and write it to disk
            const schema = config_1.ConfigSchema.buildSchema(utilities_1.ImperativeConfig.instance.loadedConfig.profiles);
            config.setSchema(schema);
            const opts = {};
            if (!user) {
                opts.populateProperties = true;
                opts.getSecureValue = this.promptForProp.bind(this);
            }
            // Build new config and merge with existing layer
            const newConfig = yield ConfigBuilder_1.ConfigBuilder.build(utilities_1.ImperativeConfig.instance.loadedConfig, opts);
            config.api.layers.merge(newConfig);
        });
    }
    /**
     * Prompts for the value of a property on the CLI. Returns null if `--prompt false`
     * argument is passed, or prompt times out, or a blank value is entered.
     * @param propName The name of the property
     * @param property The profile property definition
     */
    promptForProp(propName, property) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            // skip prompting in CI environment
            if (this.params.arguments.prompt === false || !security_1.CredentialManagerFactory.initialized) {
                return null;
            }
            // get the summary and value
            if (((_a = property.optionDefinition) === null || _a === void 0 ? void 0 : _a.description) != null) {
                propName = `${propName} (${property.optionDefinition.description})`;
            }
            const propValue = yield this.params.response.console.prompt(`${propName} - blank to skip: `, { hideText: property.secure });
            // coerce to correct type
            if (propValue && propValue.trim().length > 0) {
                if (propValue === "true")
                    return true;
                if (propValue === "false")
                    return false;
                if (!isNaN(propValue) && !isNaN(parseFloat(propValue)))
                    return parseInt(propValue, 10);
            }
            return propValue || null;
        });
    }
}
exports.default = InitHandler;
InitHandler.DEFAULT_ROOT_PROFILE_NAME = "my_profiles";
//# sourceMappingURL=init.handler.js.map